import 'dart:async';
import 'dart:io';

import 'package:bonree_flutter_plugin/src/business/bonree_method_channel.dart';
import 'package:bonree_flutter_plugin/src/business/util/logger.dart';
import 'package:flutter/cupertino.dart';

import '../business/bonree_business_interface.dart';
import 'http/bonree_http_overrides.dart';
import 'interface/bonree_agent_interface.dart';

class BonreeImpl implements Bonree {
  static Bonree? _instance;
  final BonreeBusinessInterface _business;
  Function? _errorCallback;

  ///plugin是否启动
  bool _started = false;

  factory BonreeImpl() {
    if (_instance == null) {
      _instance = BonreeImpl.private(BonreeBusinessInterface());
    }
    return _instance as BonreeImpl;
  }

  BonreeImpl.private(this._business);

  Future<void> start(Widget topLevelWidget) async {
    // If our plugin fails to intialize, allow the app to run like normal
    await runZoned(() async {
      await _start(topLevelWidget);
    }, onError: (error, stacktrace) async {
      await _business.reportCrash(error.runtimeType.toString(),
          error.message.toString(), stacktrace.toString());
      _errorCallback?.call(error, stacktrace, true);
    });
  }

  Future<void> _start(Widget topLevelWidget) async {
    try {
      if (isStarted()) {
        Logger().i("bonree_flutter_plugin is already started.");
        return;
      }
      _started = true;
      Logger().i("bonree_flutter_plugin start success");
      //开启崩溃回调监听
      WidgetsFlutterBinding.ensureInitialized();
      runApp(topLevelWidget);
      var originError = FlutterError.onError;
      FlutterError.onError = (FlutterErrorDetails details) async {
        FlutterError.dumpErrorToConsole(details, forceReport: true);
        String errorName = details.exception.runtimeType.toString();
        String reason =
            "${details.exceptionAsString()} - ${details.toStringShort()}"
                .substring(errorName.length + 2);
        _errorCallback?.call(details.exception, details.stack, false);
        if (originError != null) {
          originError(details);
        }
        await _business.reportCrash(
            errorName, reason, details.stack.toString());
      };
      //网络
      HttpOverrides.global =
          new BonreeHttpOverrides(_business, HttpOverrides.current);
    } catch (e) {}
  }

  bool isStarted() {
    return _started;
  }

  void onBonreeErrorCallback(
      callback(Object error, StackTrace stack, bool isAsync)) {
    this._errorCallback = callback;
  }

  /// 设置用户ID
  Future<void> setUserID(String userID) async {
    if (isStarted()) {
      await onMethodChannelInvoke(
          "bonree_setUserID", <String, dynamic>{"userID": userID});
    }
  }

  /// 设置附加信息
  /// @param extraInfo 附加信息为Key-Value形式
  Future<void> setExtraInfo(Map<String, dynamic> extraInfo) async {
    if (isStarted()) {
      await onMethodChannelInvoke("bonree_setExtraInfo", extraInfo);
    }
  }

  Future<void> setCustomException(
      String exceptionType, String causedBy, String errorDump) async {
    if (isStarted()) {
      await onMethodChannelInvoke(
          "bonree_setCustomException", <String, dynamic>{
        "exceptionType": exceptionType,
        "causedBy": causedBy,
        "errorDump": errorDump
      });
    }
  }

  Future<void> setCustomPageStart(String pageName, String pageAlias) async {
    if (isStarted()) {
      await onMethodChannelInvoke("bonree_setCustomPageStart",
          <String, dynamic>{"pageName": pageName, "pageAlias": pageAlias});
    }
  }

  Future<void> setCustomPageEnd(String pageName, String pageAlias) async {
    if (isStarted()) {
      await onMethodChannelInvoke("bonree_setCustomPageEnd",
          <String, dynamic>{"pageName": pageName, "pageAlias": pageAlias});
    }
  }

  Future<void> setCustomEvent(
      String eventID, String eventName, String param) async {
    if (isStarted()) {
      await onMethodChannelInvoke("bonree_setCustomEvent", <String, dynamic>{
        "eventID": eventID,
        "eventName": eventName,
        "param": param
      });
    }
  }

  Future<void> setCustomLog(String logInfo, String param) async {
    if (isStarted()) {
      await onMethodChannelInvoke("bonree_setCustomLog",
          <String, dynamic>{"logInfo": logInfo, "param": param});
    }
  }

  Future<void> setCustomMetric(
      String metricName, int metricValue, String param) async {
    if (isStarted()) {
      await onMethodChannelInvoke("bonree_setCustomMetric", <String, dynamic>{
        "metricName": metricName,
        "param": param,
        "metricValue": metricValue
      });
    }
  }

  Future<void> setCustomMethodStart(String methodName, String param) async {
    if (isStarted()) {
      await onMethodChannelInvoke(
          "bonree_setCustomMethodStart", <String, dynamic>{
        "methodName": methodName,
        "param": param,
      });
    }
  }

  Future<void> setCustomMethodEnd(String methodName, String param) async {
    if (isStarted()) {
      await onMethodChannelInvoke("bonree_setCustomMethodEnd",
          <String, dynamic>{"methodName": methodName, "param": param});
    }
  }

  @override
  Future<void> recordCustomLaunchEnd() async {
    if (isStarted()) {
      await onMethodChannelInvoke("bonree_recordCustomLaunchEnd");
    }
  }
}
